@@ -60,18 +60,19 @@ __author__ = 'Thomas Moreau (thomas.moreau.2010@gmail.com)'
 
 
 import os
+import gc
 import sys
 import types
+import struct
 import weakref
 import warnings
 import itertools
 import traceback
 import threading
+from time import time
 import multiprocessing as mp
 from functools import partial
 from pickle import PicklingError
-from time import time
-import gc
 
 from . import _base
 from .backend import get_context
@@ -80,6 +81,7 @@ from .backend.compat import wait
 from .backend.context import cpu_count
 from .backend.queues import Queue, SimpleQueue, Full
 from .backend.utils import recursive_terminate
+from .cloudpickle_wrapper import _wrap_non_picklable_objects
 
 try:
     from concurrent.futures.process import BrokenProcessPool as _BPPException
@@ -263,35 +265,16 @@ class _CallItem(object):
         return "CallItem({}, {}, {}, {})".format(
             self.work_id, self.fn, self.args, self.kwargs)
 
-    try:
-        # If cloudpickle is present on the system, use it to pickle the
-        # function. This permits to use interactive terminal for loky calls.
-        # TODO: Add option to deactivate, as it increases pickling time.
-        from .backend import LOKY_PICKLER
-        assert LOKY_PICKLER is None or LOKY_PICKLER == ""
+    def __getstate__(self):
+        return (
+            self.work_id,
+            _wrap_non_picklable_objects(self.fn),
+            [_wrap_non_picklable_objects(a) for a in self.args],
+            {k: _wrap_non_picklable_objects(a) for k, a in self.kwargs.items()}
+        )
 
-        import cloudpickle  # noqa: F401
-
-        def __getstate__(self):
-            from cloudpickle import dumps
-            if isinstance(self.fn, (types.FunctionType,
-                                    types.LambdaType,
-                                    partial)):
-                cp = True
-                fn = dumps(self.fn)
-            else:
-                cp = False
-                fn = self.fn
-            return (self.work_id, self.args, self.kwargs, fn, cp)
-
-        def __setstate__(self, state):
-            self.work_id, self.args, self.kwargs, self.fn, cp = state
-            if cp:
-                from cloudpickle import loads
-                self.fn = loads(self.fn)
-
-    except (ImportError, AssertionError) as e:
-        pass
+    def __setstate__(self, state):
+        self.work_id, self.fn, self.args, self.kwargs = state
 
 
 class _SafeQueue(Queue):
@@ -305,12 +288,17 @@ class _SafeQueue(Queue):
 
     def _on_queue_feeder_error(self, e, obj):
         if isinstance(obj, _CallItem):
-            # fromat traceback only on python3
-            pickling_error = PicklingError(
-                "Could not pickle the task to send it to the workers.")
+            # format traceback only works on python3
+            if isinstance(e, struct.error):
+                raised_error = RuntimeError(
+                    "The task could not be sent to the workers as it is too "
+                    "large for `send_bytes`.")
+            else:
+                raised_error = PicklingError(
+                    "Could not pickle the task to send it to the workers.")
             tb = traceback.format_exception(
                 type(e), e, getattr(e, "__traceback__", None))
-            pickling_error.__cause__ = _RemoteTraceback(
+            raised_error.__cause__ = _RemoteTraceback(
                 '\n"""\n{}"""'.format(''.join(tb)))
             work_item = self.pending_work_items.pop(obj.work_id, None)
             self.running_work_items.remove(obj.work_id)
@@ -318,7 +306,7 @@ class _SafeQueue(Queue):
             # case, the queue_manager_thread fails all work_items with
             # BrokenProcessPool
             if work_item is not None:
-                work_item.future.set_exception(pickling_error)
+                work_item.future.set_exception(raised_error)
                 del work_item
             self.thread_wakeup.wakeup()
         else:
@@ -394,8 +382,8 @@ def _process_worker(call_queue, result_queue, initializer, initargs,
     # set the global _CURRENT_DEPTH mechanism to limit recursive call
     global _CURRENT_DEPTH
     _CURRENT_DEPTH = current_depth
-    _REFERENCE_PROCESS_SIZE = None
-    _LAST_MEMORY_CHECK = None
+    _process_reference_size = None
+    _process_last_memory_check = None
     pid = os.getpid()
 
     mp.util.debug('Worker started with timeout=%s' % timeout)
@@ -414,7 +402,13 @@ def _process_worker(call_queue, result_queue, initializer, initargs,
                 mp.util.info("Could not acquire processes_management_lock")
                 continue
         except BaseException as e:
-            traceback.print_exc()
+            previous_tb = traceback.format_exc()
+            try:
+                result_queue.put(_RemoteTraceback(previous_tb))
+            except BaseException:
+                # If we cannot format correctly the exception, at least print
+                # the traceback.
+                print(previous_tb)
             sys.exit(1)
         if call_item is None:
             # Notify queue management thread about clean worker shutdown
@@ -434,15 +428,15 @@ def _process_worker(call_queue, result_queue, initializer, initargs,
         del call_item
 
         if _get_memory_usage is not None:
-            if _REFERENCE_PROCESS_SIZE is None:
+            if _process_reference_size is None:
                 # Make reference measurement after the first call
-                _REFERENCE_PROCESS_SIZE = _get_memory_usage(pid, force_gc=True)
-                _LAST_MEMORY_CHECK = time()
+                _process_reference_size = _get_memory_usage(pid, force_gc=True)
+                _process_last_memory_check = time()
                 continue
-            if time() - _LAST_MEMORY_CHECK > _MEMORY_CHECK_DELAY:
+            if time() - _process_last_memory_check > _MEMORY_CHECK_DELAY:
                 mem_usage = _get_memory_usage(pid)
-                _LAST_MEMORY_CHECK = time()
-                if mem_usage - _REFERENCE_PROCESS_SIZE < _MAX_MEMORY_LEAK_SIZE:
+                _process_last_memory_check = time()
+                if mem_usage - _process_reference_size < _MAX_MEMORY_LEAK_SIZE:
                     # Memory usage stays within bounds: everything is fine.
                     continue
 
@@ -450,8 +444,8 @@ def _process_worker(call_queue, result_queue, initializer, initargs,
                 # after a forced garbage collection to break any reference
                 # cycles.
                 mem_usage = _get_memory_usage(pid, force_gc=True)
-                _LAST_MEMORY_CHECK = time()
-                if mem_usage - _REFERENCE_PROCESS_SIZE < _MAX_MEMORY_LEAK_SIZE:
+                _process_last_memory_check = time()
+                if mem_usage - _process_reference_size < _MAX_MEMORY_LEAK_SIZE:
                     # The GC managed to free the memory: everything is fine.
                     continue
 
@@ -617,6 +611,9 @@ def _queue_management_worker(executor_reference,
             try:
                 result_item = result_reader.recv()
                 broken = None
+                if isinstance(result_item, _RemoteTraceback):
+                    cause = result_item.tb
+                    broken = ("A task has failed to un-serialize", cause)
             except BaseException as e:
                 tb = getattr(e, "__traceback__", None)
                 if tb is None:
@@ -861,8 +858,8 @@ class ProcessPoolExecutor(_base.Executor):
 
         if initializer is not None and not callable(initializer):
             raise TypeError("initializer must be a callable")
-        self._initializer = initializer
-        self._initargs = initargs
+        self._initializer = _wrap_non_picklable_objects(initializer)
+        self._initargs = [_wrap_non_picklable_objects(a) for a in initargs]
 
         _check_max_depth(self._context)
 
